@@ -39,11 +39,14 @@ from localstack.utils.common import (
     CaptureOutput,
     get_all_subclasses,
     get_free_tcp_port,
+    in_docker,
+    is_port_open,
     json_safe,
     last_index_of,
     long_uid,
     md5,
     now,
+    retry,
     run,
     run_safe,
     safe_requests,
@@ -52,8 +55,14 @@ from localstack.utils.common import (
     timestamp,
     to_bytes,
     to_str,
+    wait_for_port_open,
+)
+from localstack.utils.docker_utils import (
+    DOCKER_CLIENT,
+    ContainerException,
+    DockerContainerStatus,
+    PortMappings,
 )
-from localstack.utils.docker_utils import DOCKER_CLIENT, ContainerException, PortMappings
 from localstack.utils.run import FuncThread
 
 # constants
@@ -70,6 +79,9 @@ LAMBDA_API_PORT_OFFSET = 9000
 
 MAX_ENV_ARGS_LENGTH = 20000
 
+# port number used in lambci images for stay-open invocation mode
+STAY_OPEN_API_PORT = 9001
+
 INTERNAL_LOG_PREFIX = "ls-daemon: "
 
 # logger
@@ -602,6 +614,10 @@ class LambdaExecutorContainers(LambdaExecutor):
             result = e.stdout or ""
             log_output = e.stderr or ""
             error = e
+        except InvocationException as e:
+            result = e.result or ""
+            log_output = e.log_output or ""
+            error = e
         try:
             result = to_str(result).strip()
         except Exception:
@@ -783,15 +799,68 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
                 container_info.name, f"{lambda_cwd}/.", DOCKER_TASK_FOLDER
             )
 
-        return DOCKER_CLIENT.exec_in_container(
-            container_name_or_id=container_info.name,
-            command=inv_context.lambda_command,
-            interactive=True,
-            env_vars=env_vars,
-            stdin=stdin,
+        lambda_docker_ip = DOCKER_CLIENT.get_container_ip(container_info.name)
+
+        if not self._should_use_stay_open_mode(lambda_docker_ip, check_port=True):
+            LOG.debug("Using 'docker exec' to run invocation in docker-reuse Lambda container")
+            return DOCKER_CLIENT.exec_in_container(
+                container_name_or_id=container_info.name,
+                command=inv_context.lambda_command,
+                interactive=True,
+                env_vars=env_vars,
+                stdin=stdin,
+            )
+
+        inv_result = self.invoke_lambda(lambda_function, inv_context, lambda_docker_ip)
+        return (inv_result.result, inv_result.log_output)
+
+    def invoke_lambda(
+        self,
+        lambda_function: LambdaFunction,
+        inv_context: InvocationContext,
+        lambda_docker_ip=None,
+    ) -> InvocationResult:
+        full_url = self._get_lambda_stay_open_url(lambda_docker_ip)
+
+        client = aws_stack.connect_to_service("lambda", endpoint_url=full_url)
+        event = inv_context.event or "{}"
+
+        LOG.debug(f"Calling {full_url} to run invocation in docker-reuse Lambda container")
+        response = client.invoke(
+            FunctionName=lambda_function.name(),
+            InvocationType=inv_context.invocation_type,
+            Payload=to_bytes(event),
+            LogType="Tail",
         )
 
-    def _execute(self, func_arn, *args, **kwargs):
+        log_output = base64.b64decode(response.get("LogResult") or b"").decode("utf-8")
+        result = response["Payload"].read().decode("utf-8")
+
+        if "FunctionError" in response:
+            raise InvocationException(
+                "Lambda process returned with error. Result: %s. Output:\n%s"
+                % (result, log_output),
+                log_output,
+                result,
+            )
+
+        return InvocationResult(result, log_output)
+
+    def _should_use_stay_open_mode(
+        self, lambda_docker_ip: Optional[str], check_port: bool = False
+    ) -> bool:
+        """Return whether to use stay-open execution mode - if we're running in Docker, the given IP
+        is defined, and if the target API endpoint is available (optionally, if check_port is True)."""
+        should_use = lambda_docker_ip and in_docker()
+        if not should_use or not check_port:
+            return False
+        full_url = self._get_lambda_stay_open_url(lambda_docker_ip)
+        return is_port_open(full_url)
+
+    def _get_lambda_stay_open_url(self, lambda_docker_ip: str) -> str:
+        return f"http://{lambda_docker_ip}:{STAY_OPEN_API_PORT}"
+
+    def _execute(self, func_arn: str, *args, **kwargs) -> InvocationResult:
         if not LAMBDA_CONCURRENCY_LOCK.get(func_arn):
             concurrency_lock = threading.RLock()
             LAMBDA_CONCURRENCY_LOCK[func_arn] = concurrency_lock
@@ -804,7 +873,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         if config.LAMBDA_REMOVE_CONTAINERS:
             self.start_idle_container_destroyer_interval()
 
-    def cleanup(self, arn=None):
+    def cleanup(self, arn: str = None):
         if arn:
             self.function_invoke_times.pop(arn, None)
             return self.destroy_docker_container(arn)
@@ -854,8 +923,20 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
                 LOG.debug("Starting docker-reuse Lambda container: %s", container_name)
                 DOCKER_CLIENT.start_container(container_name)
+
+                def wait_up():
+                    cont_status = DOCKER_CLIENT.get_container_status(container_name)
+                    assert cont_status == DockerContainerStatus.UP
+                    if not in_docker():
+                        return
+                    # if we're executing in Docker using stay-open mode, additionally check if the target is available
+                    lambda_docker_ip = DOCKER_CLIENT.get_container_ip(container_name)
+                    if self._should_use_stay_open_mode(lambda_docker_ip):
+                        full_url = self._get_lambda_stay_open_url(lambda_docker_ip)
+                        wait_for_port_open(full_url, sleep_time=0.5, retries=8)
+
                 # give the container some time to start up
-                time.sleep(1)
+                retry(wait_up, retries=15, sleep=0.8)
 
             container_network = self.get_docker_container_network(func_arn)
             entry_point = DOCKER_CLIENT.get_image_entrypoint(docker_image)
@@ -885,6 +966,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
         network = config.LAMBDA_DOCKER_NETWORK
         additional_flags = docker_flags
+
         dns = config.LAMBDA_DOCKER_DNS
 
         mount_volumes = not config.LAMBDA_REMOTE_DOCKER
@@ -896,6 +978,15 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         if os.environ.get("HOSTNAME"):
             env_vars["HOSTNAME"] = os.environ.get("HOSTNAME")
         env_vars["EDGE_PORT"] = config.EDGE_PORT
+        command = None
+        entrypoint = "/bin/bash"
+        interactive = True
+
+        if in_docker():
+            env_vars["DOCKER_LAMBDA_STAY_OPEN"] = "1"
+            entrypoint = None
+            command = [lambda_function.handler]
+            interactive = False
 
         LOG.debug(
             "Creating docker-reuse Lambda container %s from image %s", container_name, docker_image
@@ -903,9 +994,11 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         return DOCKER_CLIENT.create_container(
             image_name=docker_image,
             remove=True,
-            interactive=True,
+            interactive=interactive,
+            detach=True,
             name=container_name,
-            entrypoint="/bin/bash",
+            entrypoint=entrypoint,
+            command=command,
             network=network,
             env_vars=env_vars,
             dns=dns,
@@ -1410,18 +1503,6 @@ class Util:
             )
         docker_tag = runtime
         docker_image = config.LAMBDA_CONTAINER_REGISTRY
-        # TODO: remove prefix once execution issues are fixed with dotnetcore/python lambdas
-        #  See https://github.com/lambci/docker-lambda/pull/218
-        lambdas_to_add_prefix = [
-            "dotnetcore2.0",
-            "dotnetcore2.1",
-            "python3.6",
-            "python3.7",
-        ]
-        if docker_image == "lambci/lambda" and any(
-            img in docker_tag for img in lambdas_to_add_prefix
-        ):
-            docker_tag = "20191117-%s" % docker_tag
         if runtime == "nodejs14.x":
             # TODO temporary fix until lambci image for nodejs14.x becomes available
             docker_image = "localstack/lambda-js"
